const CognitoUserSDK = require('./CognitoUserSDK');
const config = require('./config');
const util = require('util');
/**
 * AWS Cognito SDK is meant for client side JS. So need to mock a window.
 */
global.navigator = () => null;

function signUp(firstName, lastName, email, password, autoConfirm) {
    let sdk = new CognitoUserSDK(config);
    console.log('Sign Up:',firstName,lastName,'<',email,'>', 'auto confirm:', autoConfirm);
    
    return sdk.signUp({
        email: email,
        password: password,
        firstName: firstName,
        lastName: lastName,
        autoConfirm: (autoConfirm === 'true') ? true : false
    })
        .then((result) => {
            let user = result.user;

            let retval = {}

            if (autoConfirm === 'true') {
                let session = result.session;                
                let userId = user.username;                
                let idToken = session.idToken.jwtToken;
                let accessToken = session.accessToken.jwtToken;
                let refreshToken = session.refreshToken.token;
                let retval = {
                    userId,
                    idToken,
                    accessToken,
                    refreshToken
                }
                return Promise.resolve(util.inspect(retval, { depth: null, colors: true }));                
            }    

            return Promise.resolve(`Please check your email ${ email } for confirmation code.`);
    
        });
}

function confirmRegistration(code, email, password) {
    let sdk = new CognitoUserSDK(config);
    let vcode = code;
    if (typeof (code) === 'number') {
        vcode = code.toString();        
    }
    console.log('Confirm registration for user:', email, 'with verificaiton code:', vcode);
    let user = sdk.makeUser({ email: email });
    return sdk.confirmRegistration(user, vcode, email, password)
        .then((result) => {
            return Promise.resolve('Registration Confirmation: ', result);
        });
 
}

function resendConfirmationCode(email) {
    let sdk = new CognitoUserSDK(config);
    let user = sdk.makeUser({ email });
    return sdk.resendConfirmationCode(user)
}

function signIn(email, password) {
    let sdk = new CognitoUserSDK(config);    
    console.log('Sign In:',email);
    return sdk.signIn({ email, password })
        .then((result) => {
            let user = result.user;
            let session = result.session;
            let userId = user.username;
            let idToken = session.idToken.jwtToken;
            let accessToken = session.accessToken.jwtToken;
            let refreshToken = session.refreshToken.token;


            let retval = {
                userId,
                idToken,
                accessToken,
                refreshToken
            }
            return Promise.resolve(util.inspect(retval, { depth: null, colors: true }));
        });

}

function signOut(email) {
    let sdk = new CognitoUserSDK(config);    
    // console.log('Sign Out:',email);
    let user = sdk.makeUser({ email });
    sdk.signOut(user);
    return 'Sign Out: ' + email;
}

function deleteAccount(email, password) {
    let sdk = new CognitoUserSDK(config);    
    console.log('Delete account:',email);
    let user = sdk.makeUser({ email });

    return sdk.signIn({ email, password })
        .then((result) => {
            return sdk.deleteAccount(result.user)
                .then((result) => {
                    return Promise.resolve('Account deleted successfully');
                })
        });    
}

function forgotPassword(email) {
    let sdk = new CognitoUserSDK(config);    
    console.log('Forgot password:',email);
    let user = sdk.makeUser({ email });

    return sdk.forgotPassword(user);
}

function confirmPassword(email, code, newPassword) {
    let sdk = new CognitoUserSDK(config);    
    console.log('Confirm new password:',email);
    let user = sdk.makeUser({ email });

    let vcode = code;
    if (typeof (code) === 'number') {
        vcode = code.toString();        
    }

    return sdk.confirmPassword(user, vcode, newPassword);
}

function changePassword(email, oldPassword, newPassword) {
    let sdk = new CognitoUserSDK(config);    
    console.log('Change password for:', email, 'password:',oldPassword);
    let user = sdk.makeUser({ email });

    return sdk.signIn({ email, password: oldPassword })
        .then(function (result) {
            return sdk.changePassword(result.user, oldPassword, newPassword);
        });
}

module.exports = {
    signUp,
    signIn,
    signOut,
    deleteAccount,
    confirmRegistration,
    resendConfirmationCode,
    forgotPassword,
    confirmPassword,
    changePassword
}

